
// axi2ahb_sync_demo
module axi2ahb_sync_demo
(
    input CLK,
    input RSTN,
    // UART2
    output UART2_TXD,
    input  UART2_RXD,
    // SPI Flash Memory
    inout FLASH_SPI_CSN,
    inout FLASH_SPI_MISO,
    inout FLASH_SPI_MOSI,
    inout FLASH_SPI_CLK,
    inout FLASH_SPI_HOLDN,
    inout FLASH_SPI_WPN,
    // DDR3 Memory
    output DDR3_INIT,
    output [2:0] DDR3_BANK,
    output DDR3_CS_N,
    output DDR3_RAS_N,
    output DDR3_CAS_N,
    output DDR3_WE_N,
    output DDR3_CK,
    output DDR3_CK_N,
    output DDR3_CKE,
    output DDR3_RESET_N,
    output DDR3_ODT,
    output [13:0] DDR3_ADDR,
    output [1:0] DDR3_DM,
    inout [15:0] DDR3_DQ,
    inout [1:0] DDR3_DQS,
    inout [1:0] DDR3_DQS_N,
    // JTAG
    input TCK_IN,
    input TMS_IN,
    input TRST_IN,
    input TDI_IN,
    output TDO_OUT
);


wire CORE_CLK;
wire DDR_CLK;
wire AHB_CLK;
wire APB_CLK;
wire RTC_CLK;

wire DDR3_MEMORY_CLK;
wire DDR3_CLK_IN;
wire DDR3_LOCK;
wire DDR3_STOP;


// Gowin_PLL_AE350 instantiation
Gowin_PLL_AE350 u_Gowin_PLL_AE350
(
    .clkout0(DDR_CLK),          // 50MHz    - for test
    .clkout1(CORE_CLK),         // 800MHz
    .clkout2(AHB_CLK),          // 50MHz    - for test
    .clkout3(APB_CLK),          // 50MHz    - for test
    .clkout4(RTC_CLK),          // 10MHz
    .clkin(CLK),
    .init_clk(CLK),
    .reset(1'b0),
    .enclk0(1'b1),
    .enclk1(1'b1),
    .enclk2(1'b1),
    .enclk3(1'b1),
    .enclk4(1'b1)
);


// Gowin_PLL_DDR3 instantiation
Gowin_PLL_DDR3 u_Gowin_PLL_DDR3
(
    .lock(DDR3_LOCK),
    .clkout0(DDR3_CLK_IN),          // 50MHz
    .clkout2(DDR3_MEMORY_CLK),      // 200MHz   - for test
    .clkin(CLK),
    .init_clk(CLK),
    .reset(1'b0),
    .enclk0(1'b1),
    .enclk2(DDR3_STOP)
);


wire ae350_rstn;                    // AE350 power on and hardware reset in
wire ddr3_rstn;                     // DDR3 memory reset in
wire ddr3_init_completed;           // DDR3 memory initialized completed

assign DDR3_INIT = ddr3_init_completed;


// key_debounce instantiation
// DDR3 memory reset in key debounce
key_debounce u_key_debounce_ddr3
(
    .out(ddr3_rstn),
    .in(RSTN),
    .clk(CLK),      // 50MHz
    .rstn(1'b1)
);


// key_debounce instantiation
// AE350 power on and hardware reset in key debounce
key_debounce u_key_debounce_ae350
(
    .out(ae350_rstn),
    .in(ddr3_init_completed),
    .clk(CLK),      // 50MHz
    .rstn(1'b1)
);


// AHB slave bus
// Between AE350 and AHB2AXI_Bridge
wire [31:0] EXTS_HRDATA;
wire        EXTS_HREADYIN;
wire        EXTS_HRESP;
wire [31:0] EXTS_HADDR;
wire  [2:0] EXTS_HBURST;
wire  [3:0] EXTS_HPROT;
wire        EXTS_HSEL;
wire  [2:0] EXTS_HSIZE;
wire  [1:0] EXTS_HTRANS;
wire [31:0] EXTS_HWDATA;
wire        EXTS_HWRITE;
wire        EXTS_HCLK;
wire        EXTS_HRSTN;

// AXI bus
// Between AHB2AXI_Bridge and AXI2AHB_Sync
wire        AXI_ACLK;
wire        AXI_ARESETN;
wire [31:0] AXI_ARADDR;
wire  [1:0] AXI_ARBURST;
wire  [3:0] AXI_ARCACHE;
wire  [7:0] AXI_ARLEN;
wire        AXI_ARLOCK;
wire  [2:0] AXI_ARPROT;
wire        AXI_ARREADY;
wire  [2:0] AXI_ARSIZE;
wire        AXI_ARVALID;
wire [31:0] AXI_AWADDR;
wire  [1:0] AXI_AWBURST;
wire  [3:0] AXI_AWCACHE;
wire  [7:0] AXI_AWLEN;
wire        AXI_AWLOCK;
wire  [2:0] AXI_AWPROT;
wire        AXI_AWREADY;
wire  [2:0] AXI_AWSIZE;
wire        AXI_AWVALID;
wire        AXI_BREADY;
wire  [1:0] AXI_BRESP;
wire        AXI_BVALID;
wire [31:0] AXI_RDATA;
wire        AXI_RLAST;
wire        AXI_RREADY;
wire  [1:0] AXI_RRESP;
wire        AXI_RVALID;
wire [31:0] AXI_WDATA;
wire        AXI_WLAST;
wire        AXI_WREADY;
wire  [3:0] AXI_WSTRB;
wire        AXI_WVALID;

assign AXI_ACLK = EXTS_HCLK;
assign AXI_ARESETN = EXTS_HRSTN;


// AHB_to_AXI_Bridge_Top instantiation
AHB_to_AXI_Bridge_Top u_AHB_to_AXI_Bridge_Top
(
    // AHB bus, AE350 <--> AHB2AXI
    .AHB_HCLK(EXTS_HCLK),
    .AHB_HRESETN(EXTS_HRSTN),
    .AHB_HSEL(EXTS_HSEL),
    .AHB_HWRITE(EXTS_HWRITE),
    .AHB_HSIZE(EXTS_HSIZE),
    .AHB_HBURST(EXTS_HBURST),
    .AHB_HPROT(EXTS_HPROT),
    .AHB_HTRANS(EXTS_HTRANS),
    .AHB_HADDR(EXTS_HADDR),
    .AHB_HRDATA(EXTS_HRDATA),
    .AHB_HWDATA(EXTS_HWDATA),
    .AHB_HREADY(EXTS_HREADYIN),
    .AHB_HREADYOUT(EXTS_HREADYIN),
    .AHB_HRESP(EXTS_HRESP),
    // AXI bus, AHB2AXI <--> AXI2AHB
    .AXI_ACLK(AXI_ACLK),
    .AXI_ARESETN(AXI_ARESETN),
    .AXI_ARADDR(AXI_ARADDR),
    .AXI_ARBURST(AXI_ARBURST),
    .AXI_ARCACHE(AXI_ARCACHE),
    .AXI_ARLEN(AXI_ARLEN),
    .AXI_ARLOCK(AXI_ARLOCK),
    .AXI_ARPROT(AXI_ARPROT),
    .AXI_ARREADY(AXI_ARREADY),
    .AXI_ARSIZE(AXI_ARSIZE),
    .AXI_ARVALID(AXI_ARVALID),
    .AXI_AWADDR(AXI_AWADDR),
    .AXI_AWBURST(AXI_AWBURST),
    .AXI_AWCACHE(AXI_AWCACHE),
    .AXI_AWLEN(AXI_AWLEN),
    .AXI_AWLOCK(AXI_AWLOCK),
    .AXI_AWPROT(AXI_AWPROT),
    .AXI_AWREADY(AXI_AWREADY),
    .AXI_AWSIZE(AXI_AWSIZE),
    .AXI_AWVALID(AXI_AWVALID),
    .AXI_BREADY(AXI_BREADY),
    .AXI_BRESP(AXI_BRESP),
    .AXI_BVALID(AXI_BVALID),
    .AXI_RDATA(AXI_RDATA),
    .AXI_RLAST(AXI_RLAST),
    .AXI_RREADY(AXI_RREADY),
    .AXI_RRESP(AXI_RRESP),
    .AXI_RVAILD(AXI_RVALID),
    .AXI_WDATA(AXI_WDATA),
    .AXI_WLAST(AXI_WLAST),
    .AXI_WREADY(AXI_WREADY),
    .AXI_WSTRB(AXI_WSTRB),
    .AXI_WVALID(AXI_WVALID)
);


// AHB bus
// Between AXI2AHB and AHB_RAM
wire [31:0] haddr;
wire [1:0] htrans;
wire hready;
wire hresp;
wire hwrite;
wire [2:0] hsize;
wire [2:0] hburst;
wire [3:0] hprot;
wire [31:0] hwdata;
wire [31:0] hrdata;
wire hclk;
wire hresetn;

// Synchronous clock
assign hclk = AXI_ACLK;
assign hresetn = AXI_ARESETN;


// AXI_to_AHB_Sync_Top instantiation
AXI_to_AHB_Sync_Top u_AXI_to_AHB_Sync_Top
(
    // AHB bus, AXI2AHB <--> AHB_RAM
	.haddr(haddr),
	.htrans(htrans),
	.hready(hready),
	.hresp(hresp),
	.hwrite(hwrite),
	.hsize(hsize),
	.hburst(hburst),
	.hprot(hprot),
	.hwdata(hwdata),
	.hrdata(hrdata),
    // AXI bus, AHB2AXI <--> AXI2AHB
	.awaddr(AXI_AWADDR),
	.awlen(AXI_AWLEN), 
	.awsize(AXI_AWSIZE),
	.awburst(AXI_AWBURST),
	.awlock(AXI_AWLOCK),
	.awcache(AXI_AWCACHE),
	.awprot(AXI_AWPROT),
	.awid(4'b1),
	.awvalid(AXI_AWVALID),
	.awready(AXI_AWREADY),
	.araddr(AXI_ARADDR),
	.arlen(AXI_ARLEN),
	.arsize(AXI_ARSIZE),
	.arburst(AXI_ARBURST),
	.arlock(AXI_ARLOCK),
	.arcache(AXI_ARCACHE), 
	.arprot(AXI_ARPROT),
	.arid(4'b1), 
	.arvalid(AXI_ARVALID),
	.arready(AXI_ARREADY),
	.rid(),
	.rdata(AXI_RDATA),
	.rresp(AXI_RRESP),
	.rlast(AXI_RLAST),
	.rvalid(AXI_RVALID),
	.rready(AXI_RREADY), 
	.wdata(AXI_WDATA),
	.wstrb(AXI_WSTRB),
	.wlast(AXI_WLAST),
	.wvalid(AXI_WVALID),
	.wready(AXI_WREADY),
	.bid(),
	.bresp(AXI_BRESP),
	.bvalid(AXI_BVALID),
	.bready(AXI_BREADY),
	.clk(AXI_ACLK),
	.resetn(AXI_ARESETN)
);


// gw_ahb_ram instantiation
gw_ahb_ram u_gw_ahb_ram
(
    .AHB_HRDATA(hrdata),
    .AHB_HREADYOUT(hready),
    .AHB_HRESP(hresp),
    .AHB_HTRANS(htrans),
    .AHB_HBURST(hburst),
    .AHB_HPROT(hprot),
    .AHB_HSIZE(hsize),
    .AHB_HWRITE(hwrite),
    .AHB_HADDR(haddr),
    .AHB_HWDATA(hwdata),
    .AHB_HSEL(1'b1),
    .AHB_HCLK(hclk),
    .AHB_HRESETn(hresetn)
);


// RiscV_AE350_SOC_Top instantiation
RiscV_AE350_SOC_Top u_RiscV_AE350_SOC_Top
(
    .FLASH_SPI_CSN(FLASH_SPI_CSN),
    .FLASH_SPI_MISO(FLASH_SPI_MISO),
    .FLASH_SPI_MOSI(FLASH_SPI_MOSI),
    .FLASH_SPI_CLK(FLASH_SPI_CLK),
    .FLASH_SPI_HOLDN(FLASH_SPI_HOLDN),
    .FLASH_SPI_WPN(FLASH_SPI_WPN),
    .DDR3_MEMORY_CLK(DDR3_MEMORY_CLK),
    .DDR3_CLK_IN(DDR3_CLK_IN),
    .DDR3_RSTN(ddr3_rstn),              // DDR3 memory reset in, 0 is reset state
    .DDR3_LOCK(DDR3_LOCK),
    .DDR3_STOP(DDR3_STOP),
    .DDR3_INIT(ddr3_init_completed),    // DDR3 memory initialized completed
    .DDR3_BANK(DDR3_BANK),
    .DDR3_CS_N(DDR3_CS_N),
    .DDR3_RAS_N(DDR3_RAS_N),
    .DDR3_CAS_N(DDR3_CAS_N),
    .DDR3_WE_N(DDR3_WE_N),
    .DDR3_CK(DDR3_CK),
    .DDR3_CK_N(DDR3_CK_N),
    .DDR3_CKE(DDR3_CKE),
    .DDR3_RESET_N(DDR3_RESET_N),
    .DDR3_ODT(DDR3_ODT),
    .DDR3_ADDR(DDR3_ADDR),
    .DDR3_DM(DDR3_DM),
    .DDR3_DQ(DDR3_DQ),
    .DDR3_DQS(DDR3_DQS),
    .DDR3_DQS_N(DDR3_DQS_N),
    .TCK_IN(TCK_IN),
    .TMS_IN(TMS_IN),
    .TRST_IN(TRST_IN),
    .TDI_IN(TDI_IN),
    .TDO_OUT(TDO_OUT),
    .TDO_OE(),
    .EXTS_HRDATA(EXTS_HRDATA),
    .EXTS_HREADYIN(EXTS_HREADYIN),
    .EXTS_HRESP(EXTS_HRESP),
    .EXTS_HADDR(EXTS_HADDR),
    .EXTS_HBURST(EXTS_HBURST),
    .EXTS_HPROT(EXTS_HPROT),
    .EXTS_HSEL(EXTS_HSEL),
    .EXTS_HSIZE(EXTS_HSIZE),
    .EXTS_HTRANS(EXTS_HTRANS),
    .EXTS_HWDATA(EXTS_HWDATA),
    .EXTS_HWRITE(EXTS_HWRITE),
    .EXTS_HCLK(EXTS_HCLK),
    .EXTS_HRSTN(EXTS_HRSTN),
    .UART2_TXD(UART2_TXD),
    .UART2_RTSN(),
    .UART2_RXD(UART2_RXD),
    .UART2_CTSN(1'b0),
    .UART2_DCDN(1'b0),
    .UART2_DSRN(1'b0),
    .UART2_RIN(1'b0),
    .UART2_DTRN(),
    .UART2_OUT1N(),
    .UART2_OUT2N(),
    .CORE_CLK(CORE_CLK),
    .DDR_CLK(DDR_CLK),
    .AHB_CLK(AHB_CLK),
    .APB_CLK(APB_CLK),
    .RTC_CLK(RTC_CLK),
    .POR_RSTN(ae350_rstn),              // AE350 CPU core power on reset, 0 is reset state
    .HW_RSTN(ae350_rstn)                // AE350 hardware reset, 0 is reset state
);

endmodule